// ====================================================================
//
// MAS14_Cylinder
//
// Vortex shedding around a fixed cylinder
//
// CBP 28-02-20 Status: Devel
// CBP 27-11-21 Release. Streamplot removed.
// CBP Verified 27-11-21: 18:02
// CBP Signed off
//
// ====================================================================

BDIM flow;                 // Flow field solver
Body body;                 // Body in Flow

FloodPlot flood;           // Visualization

CirculationFinder cf;      // Identifies circulators
VectorField velocityField; // The actual flow field
float velocity;            // Velocity of the flow
boolean bRunning = true;
float dT;
float time = 0;
int n;
int nLines = 10;          // For Streamlines

void setup(){
  size(800,400);             // display window size                            
  n=(int)pow(2,7);       // Grid points
  println("Nr grid points: " + n);/// number of grid points
  Window view = new Window(n,n/2);
  
  //Try a Cylinder
  body = new CircleBody(n/8,n/4,n/12,view);  // Create a cylinder
 
  // Try an Ellipse
  //body = new EllipseBody(n/8,n/4,0.2*n/4,0.2,view);
  
  // Try and Airfoil
  // body = new NACA(n/8,n/4,32,.15,view);
  // And give it some rotation
  // body.rotate(1.0*PI/180);
  
  // Try a rectangle
  // body = new RectangleBody(n/8,n/4,n/8,n/8,view);
  
  // Change the velocity here
  velocity = 1.0;
  
  // --------------------------------------------------------------------
  
  velocityField = new VectorField(n+2,n+2,velocity,0);
  flow = new BDIM(n,n,1.,body,velocityField,0,false);   // solve for flow 
  
  flood = new FloodPlot(view);   
  flood.setLegend("vorticity",-.5,.5);        //    and its legend
  
   cf = new CirculationFinder(flow,body,view); // Add a circ finder
   cf.setAnnotate(true,1.0/(n/12));            // get it to print vals
}

void draw(){
  if(bRunning){
    cf.update();
    flow.update(body); flow.update2();   // 2-step fluid update
    dT = 1/frameRate;
    time += dT;   
  }
  flood.display(flow.u.curl());          // compute and display vorticity
  body.display();                        // display the body
  cf.display();   // display circulation

  textSize(20);
  // Some real Merlin magic follows to get a simple clock output
  text("Time = "+int((time)%1000) + "." + int((10*time)%10),700,50);  

}

 void mouseClicked() {  // stop-start updates
   bRunning = !bRunning;
 }
