// ====================================================================
//
// MAS14_Cylinder_React
//
// Vortex shedding around a fixed cylinder
//
// CBP 27-02-20 | 12-03-20 | 11-07-20 | Status: Devel
// CBP 25-11-21
// Mouse click to pause - restart
// keypress 'x' to flush data to file and exit.
// Dumps Octave file 'Data.txt'.
//
// ====================================================================

BDIM flow;                 // Flow field solver
Body body;                 // Body in Flow
FloodPlot flood;           // Visualization
//StreamPlot flood;
CirculationFinder cf;      // Identifies circulators
VectorField velocityField; // The actual flow field
float velocity;            // Velocity of the flow
boolean bRunning = true;
PVector springForce;
int n;
float time, dT;
int nLines = 50;  
PrintWriter output;
float maxY = -100;
void setup(){
  size(800,800);             // display window size                            
  n=(int)pow(2,7);       // Grid points
  println("Nr grid points: " + n);/// number of grid points
  Window view = new Window(n,n);
  
  // Choose a body
  body = new CircleBody(n/8,n/2,n/16,view);  // Create a cylinder
  //body = new NACA(n/8,n/4,32,.15,view);
  body.xfree = false;        // precent x-motion
  
  // Set thee wind speer here ================================
  velocity = 1.1;
  // =========================================================
  
  velocityField = new VectorField(n+2,n+2,velocity,0);
  //flow = new BDIM(n,n,1.,body);  //Default - cannot specify velocity
  flow = new BDIM(n,n,1.,body,velocityField,0,false);   // solve for flow 
  flood = new FloodPlot(view);                // intialize a flood plot...
  flood.setLegend("vorticity",-.5,.5);        //    and its legend

   cf = new CirculationFinder(flow,body,view); // Add a circ finder
   cf.setAnnotate(true,1.0/(n/12));            // get it to print vals
   
   springForce = new PVector(0,0);
   
   output = createWriter("VortexShed/Data.txt");
}
void draw(){
  // applied restoring spring force need to use 64 - 1 not n/2 ???
  springForce.set(0,-0.5*(body.xc.y - 63),0);
  //println(springForce);
 // println(body.xc.y);
    // body.react(springForce,0,1);
  if(bRunning){
    dT = 1/frameRate;
    time += dT;   
    // Choose between the two react. First to include the fluid
    body.react(springForce.add(body.pressForce(flow.p).mult(-1)),
              body.pressMoment(flow.p)*-(-1),flow.dt);  
    // This one to get the oscillation frequency of the m-k system
    //body.react(springForce,0,flow.dt);                
    cf.update();
    flow.update(body);
    flow.update2();   // 2-step fluid update
  }
  flood.display(flow.u.curl());          // compute and display vorticity
  body.display();                        // display the body
  cf.display();     // display circulation
  //body.displayVector(springForce.add(body.pressForce(flow.p))); 
  //body.displayVector(body.pressForce(flow.p));   
  //body.displayVector(springForce); 

  textSize(20);
  // Some real Merlin magic follows to get a simple clock output
  text("Time  = "+int((time)%1000) + "." + int((10*time)%10),600,80);  
  if((body.xc.y - 63) > maxY)
    maxY = body.xc.y - 63;
  if((body.xc.y - 63) <0)
    maxY = 0;
  
  text("dispY = "+int((maxY)%1000) + "." + int((10*maxY)%10)+int((100*maxY)%10),600,100);  
  
  output.println(""+time+" , "+body.xc.y+" , "+body.dxc.y);
  
  if(time > 1800) {
    bRunning = false;
  }
}

 void mouseClicked() {  // stop-start updates
   bRunning = !bRunning;
 }
 
 void keyPressed(){ 

  if(key == 'x') { 
    output.flush();
    output.close();
    exit();
  }  
  if(key == 'r') { 
    time = 0;
  }    
}
