// ==================================================================
//
// MAS14_EelNACA_Main
//
// CBP 11-03-20
//
// Swimming EEL (fish). Logs body-forces and displays circulation
//
// ===================================================================

BDIM flow;
FlexNACA fish; 
FloodPlot flood;
float ttime = 0;
float[] a={0,.01,-.005};

float waveSpeed;
float wavelength;
float wavenumber;
float period;
float amplitude;

boolean bRunning = true;

CirculationFinder cf;      // Identifies circulators
PrintWriter output;

void setup(){
  int n=(int)pow(2,6);
  size(1200,400);      
  Window view = new Window(3*n,n);
  ttime = 0;
  
  // Set the parameters here ------------------------------------------
  
  period = 10;
  wavelength = 11.0;
  amplitude = 0.25;
  
  // ------------------------------------------------------------------
  
  wavenumber = 2*PI/wavelength; // default wavenumber was 1.0
  waveSpeed = wavelength/period;
  
  fish = new FlexNACA(n/4,n/2,n/3,0.20,amplitude,waveSpeed,wavenumber,a,view);
  flow = new BDIM(3*n,n,0.5,fish,0.001,true);
  flood = new FloodPlot(view);
  flood.range = new Scale(-.5,.5);
  flood.setLegend("vorticity");
  flood.setColorMode(1); 
   cf = new CirculationFinder(flow,fish,view); // Add a circ finder
   cf.setAnnotate(true,1.0/(n/12));            // get it to print vals
   output = createWriter("eel/Data.txt");   
}
void draw(){
  if(bRunning) {   
    fish.update(ttime);
    flow.update(fish);
    flow.update2();    
    cf.update(); 
    ttime += flow.dt; 
  }

  flood.display(flow.u.curl());
  fish.displayVector(fish.pressForce(flow.p)); // display force
  fish.display(); 
  cf.display();   // display circulation 
  
  textSize(20);
  // Some real Merlin magic follows to get a simple clock output
  text("Time = "+int((ttime/10)%1000) + "." + int((10*ttime/10)%10),700,50);  

  PVector force = fish.pressForce(flow.p);  // pressure force on both bodies                           // time coefficient
  float forceX = force.x;                     // thrust coefficient on front
  float forceY = force.y; 
  if(bRunning)
    output.println(""+ttime/10+"  ,  "+forceX+"  ,  "+forceY);
}

 void mouseClicked() {  // stop-start updates
   bRunning = !bRunning;
 }
 
 void keyPressed(){ 
  if(key == 'r') { 
    ttime = 0;
  }
  if(key == 'x') { 
    output.flush();
    output.close();
    exit();
  }  
}
