// ====================================================================
//
// MAS14_NACA
// CBP 25-11-21
// Shows Vortex Shedding aroun airfoil for speeds and blade angle 
// CBP 27-11-21 Verified
//
// ====================================================================

BDIM flow;                 // Flow field solver
Body body;                 // Body in Flow

FloodPlot flood;           // Visualization

CirculationFinder cf;      // Identifies circulators
VectorField velocityField; // The actual flow field
float velocity;            // Velocity of the flow
boolean bRunning = true;
float dT;
float time = 0;
int n;
//int nLines = 100;          // For Streamlines
float pitchAngle;

void setup(){
  size(800,400);             // display window size                            
  n=(int)pow(2,8);       // Grid points
  println("Nr grid points: " + n);/// number of grid points
  Window view = new Window(n,n/2);
  
  // Set the parameters here ======================================
  velocity = 5.000;
  pitchAngle = 4.0;
  // ==============================================================
  
   body = new NACA(n/4,n/4,32,.15,view);
  // And give it some rotation
   body.rotate(pitchAngle*PI/180);
  
  velocityField = new VectorField(n+2,n+2,velocity,0);
  flow = new BDIM(n,n,1.0,body,velocityField,0,false);   // solve for flow 
  
  flood = new FloodPlot(view);   
  flood.setLegend("vorticity",-.5,.5);        //    and its legend
  
   cf = new CirculationFinder(flow,body,view); // Add a circ finder
   cf.setAnnotate(true,1.0/(n/12));            // get it to print vals
}

void draw(){
  // body.react(flow);
  if(bRunning){
    cf.update();
    flow.update(body); flow.update2();   // 2-step fluid update
    dT = 1/frameRate;
    time += dT;   
  }
  flood.display(flow.u.curl());          // compute and display vorticity
  body.display();                        // display the body
  cf.display();   // display circulation
  body.displayVector(body.pressForce(flow.p));
  textSize(20);
  // Some real Merlin magic follows to get a simple clock output
  text("Time = "+int((time)%1000) + "." + int((10*time)%10),600,50);  

}

 void mouseClicked() {  // stop-start updates
   bRunning = !bRunning;
 }
